/*******************************************************************************
 * Module to add/search contacts in device default address book
 *
 * @author Tomasz Scislo, Zaneta Szymanska <<ahref='mailto:z.szymanska@samsung.com'>z.szymanska@samsung.com</a>>
 *
 * ******************************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************/
var addressBook = function($) {
    try {
        var addressbook = tizen.contact.getDefaultAddressBook();
    } catch (e) {
        tlib.logger.warn("No addressbook");
    }

    return {
        /**
         * Method to search for contact
         *
         * @param contact {Object} contact to search for
         * @param callbacks {Object} contains callback functions
         * @returns
         */
        findContact : function(contact, callbacks) {
            var filterFirst = new tizen.AttributeFilter('name.firstName',
                    'CONTAINS', contact.first_name);
            var filterLast = new tizen.AttributeFilter('name.lastName',
                    'CONTAINS', contact.last_name);
            var intersection = new tizen.CompositeFilter("INTERSECTION", [
                    filterFirst, filterLast ]);
            try {
                addressbook.find(callbacks.success, callbacks.error,
                        intersection);
            } catch (e) {
                tlib.logger.err("addressbook.find() exc: " + e.message);
            }
        },

        /**
         * Adds contact to default addressBook
         *
         * @param contact
         * @returns true/false If everything goes fine returns true, otherwise
         *          false
         */
        add : function(contact) {
            try {
                addressbook.add(contact);
            } catch (e) {
                tlib.logger.err("Unable to add contact to address book: "
                        + e.message);
                return false;
            }

            return true;
        },

        /**
         * Updates contact in default addressBook
         *
         * @param contact
         * @returns
         */
        update : function(contact) {
            try {
                addressbook.update(contact);
            } catch (e) {
                tlib.logger.err("Unable to update contact: " + e.message);
            }
        },

        /**
         * Method to fetch all contacts from addressBook (only these which
         * contain phone number)
         *
         * @author Zaneta Szymanska
         *
         * @param callback {Object} contains callback functions
         * @returns
         */
        fetchAllContacts : function(callback) {
            var sortingMode = new tizen.SortMode('name.firstName', 'ASC');
            var phoneFilter = new tizen.AttributeFilter('phoneNumbers.number',
                    'CONTAINS', '');

            addressbook.find(callback.success, callback.error, phoneFilter,
                    sortingMode);
        },

        /**
         * Finds contact with specified phone number
         *
         * @author Zaneta Szymanska
         *
         * @param callback {Object} contains callback functions
         * @param number {String} specified phone number
         * @returns
         */
        findPhoneNumber : function(callback, number) {
            var phoneFilter = new tizen.AttributeFilter('phoneNumbers.number',
                    'CONTAINS', number);

            try {
                addressbook.find(callback.success, callback.error, phoneFilter);
            } catch (e) {
                tlib.logger.err("Phone number not found: " + e.message);
            }
        },

        /**
         * Creates contact from VCARD_30 data
         *
         * @author Zaneta Szymanska
         *
         * @param contactData {String} string representation of contact (vcard
         *        3.0)
         * @returns contact {Object} created contact
         */
        createContact : function(contactData) {
            var contact, tmp;

            // Create contact from VCARD 3.0
            try {
                tmp = new tizen.Contact(contactData, "VCARD_30");

                /**
                 * Actually contact created from VCARD cannot be added to
                 * address book, it usually stops application. To avoid this
                 * situation we use below code
                 */
                contact = new tizen.Contact({
                    name : new tizen.ContactName({
                        firstName : tmp.name.firstName,
                        lastName : tmp.name.lastName
                    }),
                    phoneNumbers : [ new tizen.ContactPhoneNumber(
                            tmp.phoneNumbers[0].number) ]
                });
            } catch (e) {
                tlib.logger.err('Following error occurred while converting: '
                        + e.name + ": " + e.message);
                return;
            }

            return contact;
        },

        /**
         * Removes contact from default addressbook
         *
         * @author Zaneta Szymanska
         *
         * @param contactId {Number}
         * @returns
         */
        removeContact : function(contactId) {
            addressbook.remove(contactId);
        },

        /**
         * Returns contact with specified id from default addressbook
         *
         * @author Zaneta Szymanska
         *
         * @param contactId {Number} id of desired contact
         * @returns contact {Object} contact object
         */
        get : function(contactId) {
            var contact = addressbook.get(contactId);
            return contact;
        }
    };
}($);