/*******************************************************************************
 * Module to manage sending and receiving contacts
 *
 * @author Zaneta Szymanska <<ahref='mailto:z.szymanska@samsung.com'>z.szymanska@samsung.com</a>>
 *
 * ******************************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************/

var contactsManager = function($) {
    // contains IDs of contacts to send
    var contactsToSend = [];
    // contains received contact objects
    var receivedContacts = [];

    /**
     * Invokes functions responsible for adding contact to addressbook and
     * refreshing list of contacts on main page
     */
    var addContact = function(id) {
        var cont = receivedContacts[id];

        cont.name.firstName = $("#display-fname-" + id).val();
        cont.name.lastName = $("#display-lname-" + id).val();

        var isAdded = addressBook.add(receivedContacts[id]);

        if (isAdded) {
            tlib.view.showPopup("Contact successfully added!");
            $('.add-to-addressbook:eq(' + id + ')').attr("disabled", true)
                    .button('refresh');
            // refresh list of contacts
            addressBook.fetchAllContacts({
                success : function(contacts) {
                    shareContacts.listOfContacts(contacts);
                },
                error : function(e) {
                    tlib.logger.err("fetchAllContacts fail: " + e.message);
                }
            });
        }
    };

    var createElement = function(cont, i) {
        var str = '', fName, lName;

        fName = ((cont.name.firstName !== null) ? cont.name.firstName : "");
        lName = ((cont.name.lastName !== null) ? cont.name.lastName : "");

        str += '<label for="display-fname" class="ui-input-text">First Name: </label>'
                + '<input type="text" id="display-fname-'
                + i
                + '" value="'
                + fName
                + '" />'
                + '<label for="display-lname" class="ui-input-text">Last Name: </label>'
                + '<input type="text" id="display-lname-'
                + i
                + '" value="'
                + lName
                + '" />'
                + '<label for="display-phone" class="ui-input-text">Phone number: </label>'
                + '<input type="text" id="display-phone" value="'
                + cont.phoneNumbers[0].number
                + '" />'
                + '<input type="button" id="'
                + i
                + '" value="Add to addressbook" class="add-to-addressbook"/>';

        return str;
    };

    var handleAddToAddressBook = function(id) {
        var phoneNumber = receivedContacts[id].phoneNumbers[0].number;
        var callback = {
            success : function(contacts) {
                if (contacts.length !== 0) {
                    var str = 'Phone number '
                            + phoneNumber
                            + ' already exists. Do you want to duplicate contact?';

                    if (confirm(str)) {
                        addContact(id);
                    }
                } else {
                    tlib.logger.info("Phone number " + phoneNumber
                            + " doesn't exist, contact added");
                    addContact(id);
                }
            },
            error : shareContacts.onError
        };

        addressBook.findPhoneNumber(callback, phoneNumber);
    };

    return {
        /**
         * Invoked when contact is received
         *
         * @param message {Object} NDEF Message object
         */
        handleContactReceive : function(message) {
            var str = '', cont;

            // Clear receivedContacts to contain now displayed contacts
            receivedContacts.length = 0;
            $('#received-contacts').empty();

            // Display received contacts
            for ( var i = 0; i < message.records.length; i++) {
                cont = addressBook
                        .createContact(decodeURIComponent(message.records[i].text));
                receivedContacts.push(cont);
                str += createElement(cont, i);
            }

            $('#received-contacts').append(str).trigger("create");

            $('.add-to-addressbook').unbind().click(function(event) {
                event.preventDefault();

                var id = $(this).attr("id");
                handleAddToAddressBook(id);
            });

            $.mobile.changePage('#display-page');

            $('.add-to-addressbook').button("refresh");
        },

        /**
         * Prepares contact data and invokes method to send it by NFC
         */
        sendContacts : function() {
            if (nfc.isTargetDetect()) {
                tlib.logger.info("Target is detected");

                var contactData, textNDEF, ndefRecords = [];

                for ( var i = 0; i < contactsToSend.length; i++) {
                    contactData = addressBook.get(contactsToSend[i]);

                    try {
                        textNDEF = contactData.convertToString("VCARD_30");
                        tlib.logger.info('textual representation: ' + textNDEF);
                    } catch (e) {
                        tlib.logger
                                .err('The following error occurred while converting: '
                                        + e.name + ": " + e.message);
                        return;
                    }

                    ndefRecords.push(textNDEF);
                }

                var callback = {
                    success : function() {
                        tlib.view.showPopup("Contacts were successfully sent!");
                    },
                    error : shareContacts.onError
                };

                nfc.sendTextNDEF(ndefRecords, callback);

                contactsToSend.length = [];
                $('#contacts-table').empty();
                $('#add-more-contacts').addClass('hidden');
            } else {
                tlib.view.showPopup("Target is not detected!");
            }
        },

        /**
         * Returns contactsToSend array
         *
         * @returns contactsToSend {Object} array containing IDs of contacts to
         *          send
         */
        getContactsToSend : function() {
            return contactsToSend;
        },

        /**
         * Returns receivedContacts array
         *
         * @returns receivedContacts {Object} array containing received contact
         *          objects
         */
        getReceivedContacts : function() {
            return receivedContacts;
        }
    };
}($);