/*******************************************************************************
 * Module to manage sending and receiving of messages
 *
 * @author Zaneta Szymanska <<ahref='mailto:z.szymanska@samsung.com'>z.szymanska@samsung.com</a>>
 *
 * ******************************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************/

var messageManager = function($) {
    // contains IDs of messages which you can send or save
    var checkedMessages = [];
    // contains received or searched message objects
    var displayedMesssages = [];

    return {
        /**
         * Displays list of searched or received messages
         *
         * @param messages {Object} NDEF Message object
         */
        display : function(messages) {
            tlib.logger.info(messages);
            var str = '', i = 0;

            // Remove previous items from list
            $("#messages-list >li[data-item-type=no-message]").remove();
            $("#messages-list >li[data-item-type=message]").remove();

            // if no messsages found
            if (messages.length === 0) {
                $('#send-save-button').attr("disabled", true).button('refresh');
                str += '<li class="ui-li-has-multiline" data-item-type="no-message"> No messages found </li>';
                $("#messages-list").append(str).trigger("create").listview(
                        "refresh");
            } else {
                var message, max;

                $('#send-save-button').attr("disabled", false)
                        .button('refresh');
                if ($("#send-save-button").val() === "Send") {
                    // Search messages from swiped contact
                    for (i = 0; i < messages.length; i++) {
                        var messageObject = {
                            from : messages[i].from,
                            body : messages[i].body.plainBody,
                            time : messages[i].timestamp.toString()
                        };

                        displayedMesssages.push(messageObject);
                    }
                } else {
                    // Read messages sent by other device or Event Injector
                    for (i = 0; i < messages.length; i++) {
                        var messageObject = {
                            from : messages[i].from,
                            body : messages[i].body,
                            time : messages[i].time
                        };

                        displayedMesssages.push(messageObject);
                    }
                }

                // Display list of messages
                for (i = 0; i < displayedMesssages.length; i++) {
                    message = displayedMesssages[i];
                    max = message.body.length < 15 ? message.body.length : 15;

                    // Create message item on list
                    str += '<li class="ui-li-has-multiline" data-item-type="message" data-id="'
                            + i
                            + '">'
                            + '<input type="checkbox" value="'
                            + i
                            + '" name="'
                            + i
                            + '" data-id="'
                            + i
                            + '" id="'
                            + i
                            + '" class="custom">'
                            + '<label for="'
                            + i
                            + '" style="border: 0;">'
                            + '<span class="date">'
                            + message.time.substr(8, 2)
                            + " "
                            + message.time.substr(4, 3)
                            + ",     "
                            + '</span>'
                            + '<span class="ui-li-text-sub">'
                            + message.body.substr(0, max)
                            + '</span>'
                            + '</label></li>';
                }

                var elem = $(str).unbind('swipeleft')
                        .bind(
                                'swipeleft',
                                function(e) {
                                    // Display swiped message
                                    var id = $(this).data("id");
                                    $('#message-from').val(
                                            displayedMesssages[id].from);
                                    $('#message-body').val(
                                            displayedMesssages[id].body);
                                    $.mobile.changePage('#display-message');
                                });

                $("#messages-list").append(elem).trigger("create").listview(
                        "refresh");
            }
        },

        /**
         * Sends NDEFmessage by NFC
         */
        sendMessages : function() {
            if (nfc.isTargetDetect()) {
                tlib.logger.info("Target is detected");

                if (checkedMessages.length === 0) {
                    tlib.view.showPopup("Check at least one message!");
                    return;
                }

                var smsMessages = [];

                for ( var i = 0; i < checkedMessages.length; i++) {
                    smsMessages
                            .push("sms"
                                    + JSON
                                            .stringify(displayedMesssages[checkedMessages[i]]));
                }

                nfc.sendTextNDEF(smsMessages, {
                    success : function() {
                        $.mobile.changePage('#main-page');
                        tlib.view.showPopup("The message has been sent");
                    },
                    error : shareContacts.onError
                });
            } else {
                tlib.logger.warn("Target is not detected");
                tlib.view.showPopup("Target is not detected!");
            }
        },

        /**
         * Saves message in 'Draft' folder
         */
        saveInDraftsFolder : function() {
            if (checkedMessages.length === 0) {
                tlib.logger
                        .warn("Check the message(s) you want to save in 'Draft' folder");
                tlib.view
                        .showPopup("Check the message(s) you want to save in 'Draft' folder");
                return;
            }

            var message = null, callback = {
                success : function() {
                    tlib.logger.info("Messages added to Draft Messages Folder");
                    tlib.view
                            .showPopup("Messages added to Draft Messages Folder");
                },
                error : function(e) {
                    tlib.logger.err("Cannot add message to draft folder: "
                            + e.message);
                }
            };

            for ( var i = 0; i < checkedMessages.length; i++) {
                // Create message
                message = new tizen.Message("messaging.sms", {
                    plainBody : displayedMesssages[checkedMessages[i]].body
                });

                // Save message in Drafts
                messages.addToDraft(message, callback);
            }
        },

        /**
         * Invoked when message with sms(es) is received
         *
         * @param message {Object} NDEF Message object
         */
        handleMessageReceive : function(message) {
            checkedMessages.length = 0;
            displayedMesssages.length = 0;

            var records = [];

            // Read received data to 'record' array
            for ( var i = 0; i < message.records.length; i++) {
                try {
                    records[i] = jQuery
                            .parseJSON(decodeURIComponent(message.records[i].text
                                    .slice(3)));
                } catch (e) {
                    shareContacts.onError(e);
                }

                tlib.logger.info("text: " + JSON.stringify(records[i]));
            }

            // Set '#messages-page' page before display
            var msgLabel = "Messages from: " + records[0].from;
            $('#label-msg-from').text(msgLabel.toString());

            $.mobile.changePage('#messages-page');

            $('#send-save-button').val("Save").button("refresh");

            messageManager.display(records);
        },

        /**
         * @returns checkedMessages {Object} array containing IDs of checked
         *          messages which you can send or save
         */
        getCheckedMessages : function() {
            return checkedMessages;
        },

        /**
         * @returns displayedMesssages {Object} array containing received or
         *          searched message objects
         */
        getDisplayedMesssages : function() {
            return displayedMesssages;
        }
    };
}($);