/*******************************************************************************
 * Module to turn the default NFC adapter, sends & receives NDEF messages,
 * checks if peer-to-peer target is detected
 *
 * @author Zaneta Szymanska <<ahref='mailto:z.szymanska@samsung.com'>z.szymanska@samsung.com</a>>
 *
 * ******************************************************************************************************************
 *
 * Copyright (c) 2012 Samsung Electronics All Rights Reserved.
 *
 ******************************************************************************/
var nfc = function($) {
    var nfcAdapter, nfcTarget,
    // contains callback functions: onattach, ondetach and onreceive
    callbacks;

    /**
     * Registers a callback function to be invoked when NFC peer-to-peer target
     * is detected
     */
    var detectTarget = function() {
        var onSuccess = {
            onattach : function sucAttach(target) {
                nfcTarget = target;
                nfcTarget.setReceiveNDEFListener(callbacks.onreceive, err);
                callbacks.onattach();
            },
            ondetach : function() {
                nfcTarget = null;
                callbacks.ondetach();
                nfcTarget.unsetReceiveNDEFListener();
            }
        };

        var err = function(e) {
            tlib.logger.err('Target Listen Error: ' + e.message);
        };

        try {
            nfcAdapter.setPeerListener(onSuccess, err);
        } catch (e) {
            tlib.logger.err(e.name + " : " + e.message);
        }
    };

    return {
        /**
         * Turns the default NFC adapter on the device on or off
         *
         * @param callback {Object} contains callback functions: onattach,
         *        ondetach and onreceive
         */
        init : function(callback) {
            var onPowerOnFails = function(e) {
                tlib.logger.err('PowerOn error: ' + e.name + " : " + e.message);
            };

            try {
                nfcAdapter = tizen.nfc.getDefaultAdapter();
                callbacks = callback;
                nfcAdapter.setPowered(true, detectTarget, onPowerOnFails);
            } catch (e) {
                tlib.logger.err(e.name + " : " + e.message);
                nfcAdapter = null;
            }
        },

        /**
         * Checks if target is detected
         *
         * @returns true/false {Boolean} True - if is detected, otherwise false
         */
        isTargetDetect : function() {
            if (nfcTarget) {
                return true;
            }
            return false;
        },

        /**
         * Creates and sends well-known type NDEF message
         *
         * @param recordsToSend {Object} array of objects to send
         * @param callback {Object} contains callback functions
         * @returns
         */
        sendTextNDEF : function(recordsToSend, callback) {
            var textRecords = [];
            // extra functionality, in case you use only Emulator, not device
            // content to save in the file
            var fileContent = '';

            try {
                for ( var i = 0; i < recordsToSend.length; i++) {
                    textRecords.push(new tizen.NDEFRecordText(
                            encodeURIComponent(recordsToSend[i]), 'en-US',
                            'UTF16'));
                    fileContent += i + ': '
                            + encodeURIComponent(recordsToSend[i]) + '\n';
                    tlib.logger.info("text: "
                            + encodeURIComponent(recordsToSend[i]));
                }

                var ndefMessage = new tizen.NDEFMessage(textRecords);

                nfcTarget.sendNDEF(ndefMessage, callback.success,
                        callback.error);
                {
                    // Save to file functionality
                    var filename = "nfc-data";
                    filesystem.createFile(filename);

                    var text = fileContent;
                    filesystem.writeToFile(filename, text);
                }
                tlib.logger.info("Message: " + JSON.stringify(ndefMessage));
            } catch (e) {
                tlib.logger.err("Error: " + e.message);
            }
        }
    };
}($);